#!/bin/ch
// file: acceleration.ch
// cgi code for displaying acceleration of quickreturn mech
// created: Ian Strimaitis, 5-7-2004
// modified: Matt Campbell, 4-7-2005 
#include "quickreturn.h"
#include <string.h>
#include <cgi.h>

int main()
{
   class CQuickReturn qr; 
   class CRequest Request;
   class CResponse Response;
   class CPlot plot;
   chchar *r1, *r2, *r4, *r5, *r7;
   chchar *theta1, *theta2, *omega2, *mode, *unit, *output;
   double u_r1, u_r2, u_r4, u_r5, u_r7, u_theta1, u_theta2, u_omega2;
   double data;
   double complex cmplx; 
   int numpoints = 360;
   bool u_unit;
   string_t unit_display;

   /* get user input */
   unit = Request.getForm("unit"); 
   if(!strcmp(unit, "USC"))
      u_unit = USC;
   if(!strcmp(unit, "SI"))
      u_unit = SI;

   /* get angle units, degrees or radians */
   mode = Request.getForm("mode");

   /* get link lengths */
   r1 = Request.getForm("r1");
   r2 = Request.getForm("r2");
   r4 = Request.getForm("r4");
   r5 = Request.getForm("r5");
   r7 = Request.getForm("r7");
   
   /* get input link angle, theta2; convert later */
   theta1 = Request.getForm("theta1");
   theta2 = Request.getForm("theta2");
   omega2 = Request.getForm("omega2");
   
   /* Used to choose between visual and numerical output */
   output = Request.getForm("output");
   
   /* set link lengths to be used in class */
   u_r1 = streval(r1);
   u_r2 = streval(r2);
   u_r4 = streval(r4);
   u_r5 = streval(r5);
   u_r7 = streval(r7);
   u_theta1 = streval(theta1);
   u_theta2 = streval(theta2);
   u_omega2 = streval(omega2);
   if(!strcmp(mode, "deg"))
   {
      u_theta2 = M_DEG2RAD(u_theta2);
      u_omega2 = M_DEG2RAD(u_omega2);
   }
   
   /* Set mechanism variables */
   qr.uscUnit(u_unit);
   qr.setLinks(u_r1, u_r2, u_r4, u_r5, u_r7, u_theta1);
   qr.setAngVel(u_omega2);

   /* Output for Web */
   if(!strcmp(output, "displayAcceleration"))
   {
      Response.setContentType("text/html");
      Response.begin();
      Response.title("Quick Return Acceleration Analysis");
      printf("<h3>Analysis of Slider Acceleration and Mechanism Angular Accelerations</h3>\n");
      printf("<p align=\"center\">\n");
      printf("<img src=\"/projects/mechanism/quickreturn/fig/qr_vector.png\""
             "border=\"1\" title=\"Vector &amp; Angle definitions used in calculations\""
             "alt=\"Vectors &amp; Angles\" />\n");
      printf("</p><br />\n");
      if(!strcmp(unit, "USC"))
         unit_display = "ft/s<sup>2</sup>";
      else
         unit_display = "m/s<sup>2</sup>";
      printf("<p>Slider Acceleration (a<sub>6</sub>) = %.4f %s<br />\n", qr.sliderAccel(u_theta2), unit_display);
      printf("Accelerations of points A &amp; B:\n<br />");

      cmplx = qr.getPointAccel(u_theta2, QR_POINT_A);
      printf("a<sub>A,X</sub> = %.4f %s ; ", real(cmplx), unit_display); 
      printf("a<sub>A,Y</sub> = %.4f %s ; ", imag(cmplx), unit_display);
      printf("a<sub>A</sub> = %.4f %s \n<br />", abs(cmplx), unit_display);

      cmplx = qr.getPointAccel(u_theta2, QR_POINT_B);
      printf("a<sub>B,X</sub> = %.4f %s ; ", real(cmplx), unit_display); 
      printf("a<sub>B,Y</sub> = %.4f %s ; ", imag(cmplx), unit_display);
      printf("a<sub>B</sub> = %.4f %s </p>\n\n", abs(cmplx), unit_display);

      printf("<p>Link Angular Accelerations:<br />\n");
      if(!strcmp(mode, "deg"))
      {
         data = qr.getAngAccel(u_theta2, QR_LINK_4)/M_PI_180;
         printf("&alpha;<sub>4</sub> = %.3f &deg;/s<sup>2</sup><br />\n", data);
         data = qr.getAngAccel(u_theta2, QR_LINK_5)/M_PI_180;
         printf("&alpha;<sub>5</sub> = %.3f &deg;/s<sup>2</sup></p>\n<br />\n", data);
      }
      else
      {
         printf("&alpha;<sub>4</sub> = %.3f rad/s<sup>2</sup><br />\n", qr.getAngAccel(u_theta2, QR_LINK_4));
         printf("&alpha;<sub>5</sub> = %.3f rad/s<sup>2</sup></p>\n<br />\n", qr.getAngAccel(u_theta2, QR_LINK_5));
      }
      printf("<p><a href=\"/projects/mechanism/quickreturn/acceleration.html\">"
             "Back to Acceleration Calculation page to try again.</a></p>\n");
      printf("<p><a href=\"/projects/mechanism/quickreturn/web_based.html\">"
             "Return to Web-based Interface Page to try a different analysis.</a></p>\n");
      printf("<p><a href=\"/projects/mechanism/quickreturn/\">"
             "Return to Quick Return Mechanism Home Page.</a></p>\n");
      Response.end();
   }
   if(!strcmp(output, "displayPlot"))
   {
      Response.setContentType("image/png");
      Response.begin();
      plot.outputType(PLOT_OUTPUTTYPE_STREAM, "png color");
      qr.setNumPoints(numpoints);
      qr.plotSliderAccel(&plot);
      Response.end();
   }
   if(!strcmp(output, "displayMech"))
   {
      Response.setContentType("application/x-qnm");
      Response.begin();
      qr.displayPosition(u_theta2, QANIMATE_OUTPUTTYPE_STREAM);
      Response.end();
   }
   return 0;
}

