#!/bin/ch
// file: dynamic.ch
// cgi code for displaying forces of quickreturn mech
// created: Ian Strimaitis, 5-7-2004
// modified: Matt Campbell, 4-7-2005 
#include "quickreturn.h"
#include <string.h>
#include <cgi.h>

int main()
{
   class CQuickReturn qr; 
   class CRequest Request;
   class CResponse Response;
   class CPlot plot;
   chchar *r1, *r2, *r4, *r5, *r7, *m2, *m3, *m4, *m5, *m6, *rg2, *rg4, *rg5, *ig2, *ig4, *ig5, *fl;
   chchar *theta1, *theta2, *omega2, *delta2, *delta4, *delta5, *mode, *unit, *output, *loadforce, *plottype;
   double u_r1, u_r2, u_r4, u_r5, u_r7, u_m2, u_m3, u_m4, u_m5, u_m6, u_rg2, u_rg4, u_rg5, u_ig2, u_ig4, u_ig5, u_fl;
   double u_theta1, u_theta2, u_omega2, u_delta2, u_delta4, u_delta5; 
   double data;
   double complex cmplx; 
   array double u_forces[12] = {0};
   int numpoints = 360, u_forceplot;
   bool u_unit;
   string_t unit_display;

   /* get user input */
   unit = Request.getForm("unit"); 
   if(!strcmp(unit, "USC"))
      u_unit = USC;
   if(!strcmp(unit, "SI"))
      u_unit = SI;

   /* get angle units, degrees or radians */
   mode = Request.getForm("mode");

   /* Check if there is a load force */
   loadforce = Request.getForm("loadforce");

   /* get link data */
   r1 = Request.getForm("r1");
   r2 = Request.getForm("r2");
   r4 = Request.getForm("r4");
   r5 = Request.getForm("r5");
   r7 = Request.getForm("r7");
   m2 = Request.getForm("m2");
   m3 = Request.getForm("m3");
   m4 = Request.getForm("m4");
   m5 = Request.getForm("m5");
   m6 = Request.getForm("m6");
   rg2 = Request.getForm("rg2");
   rg4 = Request.getForm("rg4");
   rg5 = Request.getForm("rg5");
   ig2 = Request.getForm("ig2");
   ig4 = Request.getForm("ig4");
   ig5 = Request.getForm("ig5");
   if(!strcmp(loadforce, "force_on"))
      fl = Request.getForm("fl");

   /* get input link angles; convert later */
   theta1 = Request.getForm("theta1");
   theta2 = Request.getForm("theta2");
   omega2 = Request.getForm("omega2");
   delta2 = Request.getForm("delta2");
   delta4 = Request.getForm("delta4");
   delta5 = Request.getForm("delta5");
   
   /* Used to choose between visual and numerical output */
   output = Request.getForm("output");
   
   /* set link lengths to be used in class */
   u_r1 = streval(r1);
   u_r2 = streval(r2);
   u_r4 = streval(r4);
   u_r5 = streval(r5);
   u_r7 = streval(r7);
   u_m2 = streval(m2); 
   u_m3 = streval(m3);
   u_m4 = streval(m4);
   u_m5 = streval(m5);
   u_m6 = streval(m6);
   u_rg2 = streval(rg2);
   u_rg4 = streval(rg4);
   u_rg5 = streval(rg5);
   u_ig2 = streval(ig2);
   u_ig4 = streval(ig4);
   u_ig5 = streval(ig5);
   if(!strcmp(loadforce, "force_on"))
      u_fl = streval(fl);
   u_theta1 = streval(theta1);
   u_theta2 = streval(theta2);
   u_omega2 = streval(omega2);
   u_delta2 = streval(delta2);
   u_delta4 = streval(delta4);
   u_delta5 = streval(delta5);
   if(!strcmp(mode, "deg"))
   {
      u_theta2 *= M_PI_180;
      u_omega2 *= M_PI_180;
      u_delta2 *= M_PI_180;
      u_delta4 *= M_PI_180;
      u_delta5 *= M_PI_180;
   }
   
   /* Set mechanism variables */
   qr.uscUnit(u_unit);
   qr.setLinks(u_r1, u_r2, u_r4, u_r5, u_r7, u_theta1);
   qr.setAngVel(u_omega2);
   qr.setGravityCenter(u_rg2, u_rg4, u_rg5, u_delta2, u_delta4, u_delta5);
   qr.setInertia(u_ig2, u_ig4, u_ig5);
   qr.setMass(u_m2, u_m3, u_m4, u_m5, u_m6);
   qr.setForce(u_fl);

   /* Output for Web */
   if(!strcmp(output, "displayForces"))
   {
      Response.setContentType("text/html");
      Response.begin();
      Response.title("Quick Return Dynamic Analysis");
      printf("<h3>Analysis of Mechanism Forces and Input Torque</h3>\n");
      printf("<p align=\"center\">\n");
      printf("<img src=\"/projects/mechanism/quickreturn/fig/qr_vector.png\""
             "border=\"1\" title=\"Vector &amp; Angle definitions used in calculations\""
             "alt=\"Vectors &amp; Angles\" />\n");
      printf("</p><br />\n");
      if(!strcmp(unit, "USC"))
         unit_display = "lbs";
      else
         unit_display = "N";

      if(!strcmp(unit, "USC"))
         printf("<p>Input Torque (T<sub>s</sub>) = %.4f %s<br /><br />\n", qr.getRequiredTorque(u_theta2), "ft-lb");
      else
         printf("<p>Input Torque (T<sub>s</sub>) = %.4f %s<br /><br />\n", qr.getRequiredTorque(u_theta2), "N-m");

      qr.getForces(u_theta2, u_forces);
      printf("Forces on Mechanism:<br />\n");
      printf("<table width=\"50%%\">\n");
      printf("<tr><td>Force Name</td>    <td>Value (%s)</td></tr>\n", unit_display);
      printf("<tr><td>F12x</td>          <td>%.4f</td></tr>\n", u_forces[0]);
      printf("<tr><td>F12y</td>          <td>%.4f</td></tr>\n", u_forces[1]);
      printf("<tr><td>F23x</td>          <td>%.4f</td></tr>\n", u_forces[2]);
      printf("<tr><td>F23y</td>          <td>%.4f</td></tr>\n", u_forces[3]);
      printf("<tr><td>F14x</td>          <td>%.4f</td></tr>\n", u_forces[4]);
      printf("<tr><td>F14y</td>          <td>%.4f</td></tr>\n", u_forces[5]);
      printf("<tr><td>F34</td>           <td>%.4f</td></tr>\n", u_forces[6]);
      printf("<tr><td>F45x</td>          <td>%.4f</td></tr>\n", u_forces[7]);
      printf("<tr><td>F45y</td>          <td>%.4f</td></tr>\n", u_forces[8]);
      printf("<tr><td>F56x</td>          <td>%.4f</td></tr>\n", u_forces[9]);
      printf("<tr><td>F56y</td>          <td>%.4f</td></tr>\n", u_forces[10]);
      printf("<tr><td>F16y</td>          <td>%.4f</td></tr>\n", u_forces[11]);
      printf("</table>\n");
      printf("<br /><br />\n");


      printf("<p><a href=\"/projects/mechanism/quickreturn/dynamic.html\">"
             "Back to Force Calculation page to try again.</a></p>\n");
      printf("<p><a href=\"/projects/mechanism/quickreturn/web_based.html\">"
             "Return to Web-based Interface Page to try a different analysis.</a></p>\n");
      printf("<p><a href=\"/projects/mechanism/quickreturn/\">"
             "Return to Quick Return Mechanism Home Page.</a></p>\n");
      Response.end();
   }
   if(!strcmp(output, "displayPlot"))
   {
      /* Check if which plot was chosen */
      plottype = Request.getForm("plottype");
      switch(plottype)
      {
         case "f12x":
            u_forceplot = F12X;
            break;
         case "f12y":
            u_forceplot = F12Y;
            break;
         case "f12mag":
            u_forceplot = MAG_F12;
            break;
         case "f23x":
            u_forceplot = F23X;
            break;
         case "f23y":
            u_forceplot = F23Y;
            break;
         case "f23mag":
            u_forceplot = MAG_F23;
            break;
         case "f14x":
            u_forceplot = F14X;
            break;
         case "f14y":
            u_forceplot = F14Y;
            break;
         case "f14mag":
            u_forceplot = MAG_F14;
            break;
         case "f34mag":
            u_forceplot = MAG_F34;
            break;
         case "f45x":
            u_forceplot = F45X;
            break;
         case "f45y":
            u_forceplot = F45Y;
            break;
         case "f45mag":
            u_forceplot = MAG_F45;
            break;
         case "f56x":
            u_forceplot = F56X;
            break;
         case "f56y":
            u_forceplot = F56Y;
            break;
         case "f56mag":
            u_forceplot = MAG_F56;
            break;
         case "f16y":
            u_forceplot = F16Y;
            break;
      }

      Response.setContentType("image/png");
      Response.begin();
      plot.outputType(PLOT_OUTPUTTYPE_STREAM, "png color");
      qr.setNumPoints(numpoints);
      if(!strcmp(plottype, "torque"))
         qr.plotTorque(&plot);
      else
         qr.plotForce(u_forceplot, &plot);
      Response.end();
   }
   if(!strcmp(output, "displayMech"))
   {
      Response.setContentType("application/x-qnm");
      Response.begin();
      qr.displayPosition(u_theta2, QANIMATE_OUTPUTTYPE_STREAM);
      Response.end();
   }
   return 0;
}

